
#include <Wire.h> // Library for I2C communication
#include <LiquidCrystal_I2C.h> // Library for LCD
#include <SparkFun_PHT_MS8607_Arduino_Library.h>
#include "HX711.h"
#include <EEPROM.h>

// Wiring: SDA(blue) pin is connected to A4 and SCL pin to A5.
//oxygen sensor to A0, DOUT to 2 and clock to 3

LiquidCrystal_I2C lcd = LiquidCrystal_I2C(0x27, 16, 2); // Change to (0x27,20,4) for 20x4 LCD.

#define calibration_factor 5138
#define zero_factor 178194

long now = millis();
float Vout =0;
int button = 1;
int button2 = 1;
int addr = 0;
float oldHumidity = 0;
float oldO2 = 0;
long stored_time = 0;
int a,b,c,d = 0;
byte value;
int address = 0;
int diff = 1;

const float VRefer = 5;
    
const int pinAdc   = A0;
const int LOADCELL_DOUT_PIN = 2;
const int LOADCELL_SCK_PIN = 3;
const int buttonPin = 4;
const int relay = 5;
const int ground = 11;
const int buttonPin2 = 12;


MS8607 barometricSensor;
HX711 scale;

byte force[] = {
  B00100,
  B00100,
  B00100,
  B00100,
  B10101,
  B01110,
  B00100,
  B00100
};

byte pressured[] = {
  B00100,
  B10101,
  B01010,
  B00101,
  B10100,
  B01010,
  B10101,
  B00100
};

byte temp[] = {
  B00110,
  B01001,
  B01001,
  B00110,
  B00000,
  B00000,
  B00000,
  B00000
};

byte humid[] = {
  B00000,
  B11000,
  B11001,
  B00010,
  B00100,
  B01000,
  B10011,
  B00011
};


void setup() {
  pinMode(buttonPin, INPUT_PULLUP);
  pinMode(relay, OUTPUT);
  pinMode(buttonPin2, INPUT_PULLUP);
  pinMode(ground, OUTPUT);
  digitalWrite(ground, LOW);
  digitalWrite(relay, LOW);
  /////////////////////////////////////////////////////////////////
  Serial.begin(9600);
  lcd.init();
  lcd.backlight();
  lcd.createChar(0,force);
  lcd.createChar(1,pressured);
  lcd.createChar(2,temp);
  lcd.createChar(3, humid);
  /////////////////////////////////////////////////////////////////
  Wire.begin();
/////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////
  if (barometricSensor.begin() == false)
  {
    Serial.println("MS8607 sensor did not respond. Trying again...");
    if (barometricSensor.begin() == false)
    {
      Serial.println("MS8607 sensor did not respond. Please check wiring.");
      while (1)
        ;
    }
  }
   int err = barometricSensor.set_humidity_resolution(MS8607_humidity_resolution_12b); // 12 bits
  if (err != MS8607_status_ok)
  {
    Serial.print("Problem setting the MS8607 sensor humidity resolution. Error code = ");
    Serial.println(err);
    Serial.println("Freezing.");
    while (1)
      ;
  }

  // Turn the humidity sensor heater OFF
  // The TE examples say that get_compensated_humidity and get_dew_point will only work if the heater is OFF
  err = barometricSensor.disable_heater();
  if (err != MS8607_status_ok)
  {
    Serial.print("Problem disabling the MS8607 humidity sensor heater. Error code = ");
    Serial.println(err);
    Serial.println("Freezing.");
    while (1)
      ;
  }
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  scale.begin(LOADCELL_DOUT_PIN, LOADCELL_SCK_PIN);
  
  Serial.println("Before setting up the scale:");
  Serial.print("read: \t\t");
  Serial.println(scale.read());      // print a raw reading from the ADC

  Serial.print("read average: \t\t");
  Serial.println(scale.read_average(20));   // print the average of 20 readings from the ADC

  Serial.print("get value: \t\t");
  Serial.println(scale.get_value(5));   // print the average of 5 readings from the ADC minus the tare weight (not set yet)

  Serial.print("get units: \t\t");
  Serial.println(scale.get_units(5), 1);  // print the average of 5 readings from the ADC minus tare weight (not set) divided
            // by the SCALE parameter (not set yet)

  scale.set_scale(calibration_factor);                      // this value is obtained by calibrating the scale with known weights; see the README for details
  scale.set_offset(zero_factor);

  Serial.println("After setting up the scale:");

  Serial.print("read: \t\t");
  Serial.println(scale.read());                 // print a raw reading from the ADC

  Serial.print("read average: \t\t");
  Serial.println(scale.read_average(20));       // print the average of 20 readings from the ADC

  Serial.print("get value: \t\t");
  Serial.println(scale.get_value(5));    // print the average of 5 readings from the ADC minus the tare weight, set with tare()

  Serial.print("get units: \t\t");
  Serial.println(scale.get_units(5), 1);        // print the average of 5 readings from the ADC minus tare weight, divided
            // by the SCALE parameter set with set_scale

  Serial.println("Readings:");
  /////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////
  
}

void loop() {

  //////////////TARE FUNCTION//////////////////////////////////
  float firstWeight;
  button = digitalRead(buttonPin);
  if (button == 0){
    lcd.clear();
    lcd.setCursor(2,0);
    lcd.print("Tare Set");
    scale.tare();
    delay(2000);
    lcd.clear();

    for (int i = 0 ; i < EEPROM.length() ; i++) 
    {
    EEPROM.write(i, 0);
    }
  
    button = 1;
    while (button == 1) {
      lcd.setCursor(2,0);
      lcd.print("load weed");
      lcd.setCursor(2,1);
      lcd.print("& hit button");
      button = digitalRead(buttonPin);
      a = 1;
    } 
    delay(5000);
   firstWeight = scale.get_units();
   delay(500);
   EEPROM.write(addr, abs((int)firstWeight));
   }
  //////////////////////////////////////////////////////////////////////////////
 /////////////SECOND BUTTON//////////////////////////////////////////////////////////
  button2 = digitalRead(buttonPin2);
  if (button2 == 0){

    button2 = 1;
    while (button2 == 1) {
    lcd.clear();
    lcd.setCursor(0,0);
    lcd.print("ADDY:");
    lcd.setCursor(6,0);
    address = address + 1;
  if (address == EEPROM.length()) {
    address = 1;
  }
    lcd.print(address);
    lcd.setCursor(0,1);
    lcd.print("VALUE:");
    lcd.setCursor(6,1);
    value = EEPROM.read(address);
    lcd.print(value);
    delay(1000);
    button2 = digitalRead(buttonPin2);
    if (value == 0)
    {
      button2 = 0;
    }
    } 
  }
  //////////////////////////////////////////////////////////////////////////////////////////
Serial.print("firstWeight ");
Serial.println((int)firstWeight);
Serial.println("EEPROM ");
Serial.println(EEPROM.read(addr));

  Vout = readO2Vout();
  float humidity = barometricSensor.getHumidity();
  float temperature = barometricSensor.getTemperature();
  float pressure = barometricSensor.getPressure();
  float O2 = readConcentration();
  /////////////////////////
  lcd.clear();
  lcd.setCursor(0, 0); 
  lcd.write(0);
  lcd.setCursor(1, 0); 
  lcd.print(scale.get_units(), 1);
  /////////////////////////
  lcd.setCursor(7,0);
  lcd.print("OXY:");
  lcd.setCursor(11,0);
  lcd.print(readConcentration(),1);
  /////////////////////////
  lcd.setCursor(0,1);
  lcd.write(1);
  lcd.setCursor(1,1);
  lcd.print(pressure,0);
  //////////////////////////
  lcd.setCursor(6,1);
  lcd.write(2);
  lcd.setCursor(7,1);
  lcd.print(temperature,0);
  ///////////////////////////
  lcd.setCursor(11,1);
  lcd.print(humidity,1);
  lcd.setCursor(15,1);
  lcd.write(3);
////////////////////////////////////////////////////
int minutes = ((millis() - stored_time)/1000)/60;
int diHumid = (humidity - oldHumidity);
int diO2 = (O2 - oldO2);
///////////Main FUNCTION////////////////

if (a == 1)                  //weed has been loaded
{
  if (b == 0)                 //start timer
  {
    stored_time = millis();
    b = 1;
  }
  if (minutes == 5 && c == 0 && d == 0) //get a value for humidity
  {
    oldHumidity = humidity;
    oldO2 = O2;
    c = 1; 
  }
  if (minutes == 10) //test conditions
  {
    if (abs(diHumid) <= 2) //stable humidity or reset timer
    {
      ////////////////
    addr = addr + 1;
    if (addr == EEPROM.length()) {
    addr = 1;}
    EEPROM.write(addr,(int)humidity);
    //////////////// 
    ////////////////
    addr = addr + 1;
    if (addr == EEPROM.length()) {
    addr = 1;}
    EEPROM.write(addr,(int)diO2);
    //////////////// 
      digitalWrite(relay, HIGH);
      d = 1;               //the pump program has started
      stored_time = millis();         //timer reset is too slow
    } else { b = 0; }      //reset the timer and check again
  }

  if (d == 1) //the pump is on
  {
       if (pressure < 980) //turn off relay
       {
          digitalWrite(relay, LOW);
       }
       if (pressure > 990) //turn on relay
       {
          digitalWrite(relay, HIGH);
       }
    if (minutes == 5)
    {
      d = 0;
      b = 0;
      c = 0;
      digitalWrite(relay, LOW);
    }
  }
  Serial.print("minutes :");
  Serial.println(minutes);
  Serial.print("diHumid: ");
  Serial.println(diHumid);
  Serial.print("B,C,D: ");
  Serial.print(b);
  Serial.print(",");
  Serial.print(c);
  Serial.print(",");
  Serial.println(d);
  
} 
///////////////////////////////////////////////////////////
  

  scale.power_down();              // put the ADC in sleep mode
  delay(2000);
  scale.power_up();


}

float readO2Vout()
{
    long sum = 0;
    for(int i=0; i<32; i++)
    {
        sum += analogRead(pinAdc);
    }
 
    sum >>= 5;
 
    float MeasuredVout = sum * (VRefer / 1023.0);
    return MeasuredVout;
}
 
float readConcentration()
{
    // Vout samples are with reference to 3.3V
    float MeasuredVout = readO2Vout();
 
    //float Concentration = FmultiMap(MeasuredVout, VoutArray,O2ConArray, 6);
    //when its output voltage is 2.0V,
    float Concentration = MeasuredVout * 0.21 / 2.0;
    float Concentration_Percentage=Concentration*100;
    return Concentration_Percentage;
}
